#!/bin/bash
#******************************************************************************
# Copyright (C), 2020-2020, Huawei Tech. Co., Ltd.
# File : install.sh
# install driver
# Date  : 2020/06/16
#******************************************************************************

mkjson(){
    firmware_name=$(tail -2 ${work_dir}/result.txt | grep firm | awk -F '"' '{print $4}')
    fir_result=$(tail -2 ${work_dir}/result.txt | grep firm | awk -F '"' '{print $8}')
    fir_AM=$(tail -2 ${work_dir}/result.txt | grep firm | awk -F '"' '{print $16}')
    driver_name=$(tail -2 ${work_dir}/result.txt | grep driver | awk -F '"' '{print $4}')
    driver_result=$(tail -2 ${work_dir}/result.txt | grep driver | awk -F '"' '{print $8}')
    driver_AM=$(tail -2 ${work_dir}/result.txt | grep driver | awk -F '"' '{print $16}')

    echo "{
    \"data\":[
        { \"File\":\"${firmware_name}\" , \"Result\":\"${fir_result}\" , \"ErrorInfo\":\"NA\" , \"ActiveMode\":\"${fir_AM}\" },
        { \"File\":\"${driver_name}\" , \"Result\":\"${driver_result}\" , \"ErrorInfo\":\"NA\" , \"ActiveMode\":\"${driver_AM}\" }
    ]
}" > result.json 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
}

var_init(){
    FILE=${LINK:-$0}
    work_dir="."
    progress_num=1
    ddri_name=npu
    err_count=0
}

#*****************************************************************************
# Description  : 用户不存在则创建一个默认用户
#*****************************************************************************
check_user(){
    local user_group=""
    local user_name=""
    local default_group="HwHiAiUser"
    local default_name="HwHiAiUser"

    # create group if not exists
    user_group=$(cat /etc/ascend_install.info | grep ^"UserGroup" | awk -F "=" '{print $2}')
    if [ -z "${user_group}" ];then
        groupadd ${default_group}
    fi

    # create user if not exists
    user_name=$(cat /etc/ascend_install.info | grep ^"UserName" | awk -F "=" '{print $2}')
    if [ -z "${user_name}" ];then
        useradd -g ${default_group} -d /home/${default_name} -m ${default_name} -s /bin/bash
    fi
}

#*****************************************************************************
# Description  : install firmware
#*****************************************************************************
install_firmware(){
   firmware_install_num=1
   firmware_name_install=$(ls *npu-firmware*.run)

   ./*npu-firmware*.run $1 --quiet
   ret=$?
   if [ $ret -eq 0 ] || [ $ret -eq 2 ];then
        echo "\"Total\":\"${firmware_install_num}\",\"Process\":\"${progress_num}\",\"DriverName\":\"${ddri_name}\",\"File\":\"${firmware_name_install}\"" >> "${work_dir}/progress.txt" 2>> "${work_dir}/log_install_failed.log" 2>/dev/null
        if [ $ret -eq 0 ];then
            echo "\"File\":\"${firmware_name_install}\",\"Result\":\"success\",\"ErrorInfo\":\"NA\",\"ActiveMode\":\" \"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
        else
            echo "\"File\":\"${firmware_name_install}\",\"Result\":\"success\",\"ErrorInfo\":\"NA\",\"ActiveMode\":\"ResetOS\"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
        fi
        err_count=0
    else
        echo "\"Total\":\"${firmware_install_num}\",\"Process\":\"${progress_num}\",\"DriverName\":\"${ddri_name}\",\"File\":\"${firmware_name_install}\"" >> "${work_dir}/progress.txt" 2>> "${work_dir}/log_install_failed.log" 2>/dev/null
        echo "\"File\":\"${firmware_name_install}\",\"Result\":\"failed\",\"ErrorInfo\":\"NA\",\"ActiveMode\":\" \"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
        err_count=1
    fi
}

#*****************************************************************************
# Description  : install driver
#*****************************************************************************
install_driver(){
    local ret
    driver_install_num=1
    driver_name_install=$(ls *npu-driver*.run)


    # 驱动安装返回值0表示：安装成功立即生效；返回值2表示：安装成功需要reboot生效；
    ./*npu-driver*.run $1 --quiet
    ret=$?
    if [ $ret -ne 0 ] && [ $ret -ne 2 ];then        #安装失败时进行重试
        ./*npu-driver*.run $1 --quiet
        ret=$?
    fi
    if [ $ret -eq 0 ] || [ $ret -eq 2 ];then
        echo "\"Total\":\"${driver_install_num}\",\"Process\":\"${progress_num}\",\"DriverName\":\"${ddri_name}\",\"File\":\"${driver_name_install}\"" >> "${work_dir}/progress.txt" 2>> "${work_dir}/log_install_failed.log" 2>/dev/null
        if [ $ret -eq 0 ];then
            echo "\"File\":\"${driver_name_install}\",\"Result\":\"success\",\"ErrorInfo\":\"NA\",\"ActiveMode\":\"Immediately\"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
        else
            echo "\"File\":\"${driver_name_install}\",\"Result\":\"success\",\"ErrorInfo\":\"NA\",\"ActiveMode\":\"ResetOS\"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
            err_count=0
        fi
    else
        echo "\"Total\":\"${driver_install_num}\",\"Process\":\"${progress_num}\",\"DriverName\":\"${ddri_name}\",\"File\":\"${driver_name_install}\"" >> "${work_dir}/progress.txt" 2>> "${work_dir}/log_install_failed.log" 2>/dev/null
        echo "\"File\":\"${driver_name_install}\",\"Result\":\"failed\",\"ErrorInfo\":\"NA\",\"ActiveMode\":\" \"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
        err_count=1
    fi
}

#****************************************************************************************************************
# 1.check whether the character-device of the driver is used in Docker based on the Docker image information,
#   if it is, the function will return 1, otherwise, it will return 0.
#****************************************************************************************************************
checkUserDocker() {
    local dev_files="/dev/davinci_manager /dev/devmm_svm /dev/hisi_hdc"
    local dev_files2=`find /dev -name "davinci[0-9]*"`
    local davinci_files=(${dev_files[*]} ${dev_files2[*]})
    local f_file
    local docker_dev
    local docker_list
    for f_file in ${davinci_files[*]}
    do
        docker >> /dev/null 2>&1
        if [ $? -eq 0 ];then
            docker_list=`docker ps -q 2> /dev/null`
            if [ -n "${docker_list}" ]; then
                docker_dev=`docker inspect $(docker ps -q) | grep "$f_file"`
                if [ -n "${docker_dev}" ]; then
                    echo "[ERROR]"$f_file" has used by docker"
                    return 1
                fi
            fi
        fi
    done
    return 0
}

#****************************************************************************************************************
# 1.check whether any process is still calling the kernel-mode ko based on the character-device of the driver,
#   if it is, the function will return 1, otherwise, it will return 0.
# 2.check whether the character-device of the driver is used in Docker based on the Docker image information,
#   if it is, the function will return 1, otherwise, it will return 0.
# 3. this check funtion is customized for HUAWEI CLOUD.
#****************************************************************************************************************
checkUserProcess() {
    local dev_files="/dev/davinci_manager /dev/devmm_svm /dev/hisi_hdc"
    local dev_files2=`find /dev -name "davinci[0-9]*"`
    local davinci_files=(${dev_files[*]} ${dev_files2[*]})
    local f_file
    local apps
    local app
    local docker_dev
    local docker_list

    if which fuser >& /dev/null; then
        for f_file in ${davinci_files[*]}
        do
            apps=`fuser -uv "$f_file" 2> /dev/null`
            if [ -n "${apps}" ]; then
                echo "[ERROR]"$f_file" has user process: ""$apps"
                for app in $apps
                do
                    ps -ef| grep -w "$app"| grep -v "grep"
                done
                return 1
        fi
        done
    else
        for f_file in ${davinci_files[*]}
        do
            apps=`ls -l /proc/*/fd |egrep "fd:|"$f_file""|grep "$f_file" -B1|grep "fd:"|sed 's#/fd:##;s#/proc/##'|head`
            if [ -n "${apps}" ]; then
                echo $f_file" has user process: ""$apps"
                for app in $apps
                do
                    ps -ef| grep -w "$app"| grep -v "grep"
                done
                return 1
            fi
        done
    fi

    checkUserDocker || return 1
    return 0
}

chipTypeIs910() {
    local chip_type_result=$(lspci | grep d801)
    if [ "$chip_type_result" = "" ]; then
        # 如果不是910 返回0
        return 0;
    fi

    return 1;
}

writeLog() {
    driver_install_num=1
    driver_name_install=$(ls *npu-driver*.run)
    firmware_install_num=1
    firmware_name_install=$(ls *npu-firmware*.run)
    echo "stop upgrade. Please checkout param or stop process."
    echo "\"Total\":\"${firmware_install_num}\",\"Process\":\"${progress_num}\",\"DriverName\":\"${ddri_name}\",\"File\":\"${firmware_name_install}\"" >> "${work_dir}/progress.txt" 2>> "${work_dir}/log_install_failed.log" 2>/dev/null
    echo "\"File\":\"${firmware_name_install}\",\"Result\":\"failed\",\"ErrorInfo\":\"The device node is occupied.\",\"ActiveMode\":\" \"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
    echo "\"Total\":\"${driver_install_num}\",\"Process\":\"${progress_num}\",\"DriverName\":\"${ddri_name}\",\"File\":\"${driver_name_install}\"" >> "${work_dir}/progress.txt" 2>> "${work_dir}/log_install_failed.log" 2>/dev/null
    echo "\"File\":\"${driver_name_install}\",\"Result\":\"failed\",\"ErrorInfo\":\"The device node is occupied.\",\"ActiveMode\":\" \"" >> "${work_dir}/result.txt" 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
    echo "{
    \"data\":[
        { \"File\":\"${firmware_name_install}\" , \"Result\":\"failed\" , \"ErrorInfo\":\"The device node is occupied.\" , \"ActiveMode\":\" \" },
        { \"File\":\"${driver_name_install}\" , \"Result\":\"failed\" , \"ErrorInfo\":\"The device node is occupied.\" , \"ActiveMode\":\" \" }
    ]
}" > result.json 2>>"${work_dir}/log_install_failed.log" 2>/dev/null
    sync
    sync
}

#*****************************************************************************
# Description  : main function
#*****************************************************************************
install_script_path=$(dirname $(readlink -f $0))
chmod +x ${install_script_path}/*
var_init

case "$1" in
    install)
        #get dir
        if [ -n "$3" ] && [ "$3" == "dir" ]; then
            work_dir="$4"
            mkdir -p ${work_dir}
        fi

        if [ -n "$2" ]&& [ "$2" == "all" ]; then

            # 检查是否带--strict 参数, 如果是则去检查是否有容器或者进程占用设备节点。如果有，则退出安装
            # 这个检查专供华为云使用，避免出现占用，导致ko卸载不掉，出现升级成功，要求reboot的情况
            # --strict 参数 只有在最后才生效，才进行检查
            # 目前只有910产品这参数才有效
            chipTypeIs910
            if [ $? -eq 1 ]; then
                args=$#
                if [ -n "${!args}" ] && [ "${!args}" == "--strict" ]; then
                    checkUserProcess
                    if [ $? -eq 1 ]; then
                        writeLog
                        exit 1
                    fi
                fi
            fi

            pushd ${install_script_path}
            # 检查环境中是否已经安装过驱动包，有驱动则进行升级,先固件再驱动；没有驱动进行安装，需要先安装驱动引导固件升级安装
            lsmod | grep drv_pcie_hdc_host
            if [ $? -eq 0 ];then
                FirmWare=$(cat /etc/ascend_install.info | grep ^"Firmware_Install_Path_Param")
                if [ "${FirmWare}" == "" ];then
                    install_firmware  --run
                else
                    install_firmware --upgrade
                fi
                install_driver --upgrade
            else
                check_user

                install_driver  --run
                FirmWare=`cat /etc/ascend_install.info | grep ^"Firmware_Install_Path_Param"`
                if [ "${FirmWare}" == "" ];then
                    install_firmware  --run
                else
                    install_firmware --upgrade
                fi
            fi
            popd
        fi
        mkjson
        sync
        sync
        if [ ${err_count} -ne 0 ]
        then
            exit 1
        else
            exit 0
        fi
    ;;
    *)
    ;;
esac
